@@ -100,19 +100,15 @@ npcdist.condbandwidth <-
       stop("length of bandwidth vector does not match number of columns of 'tydat'")
 
     xccon = unlist(lapply(txdat[,bws$ixcon, drop = FALSE],class))
-    if ((any(bws$ixcon) && !all((xccon == class(integer(0))) | (xccon == class(numeric(0))))) ||
-        (any(bws$ixord) && !all(unlist(lapply(txdat[,bws$ixord, drop = FALSE],class)) ==
-                                class(ordered(0)))) ||
-        (any(bws$ixuno) && !all(unlist(lapply(txdat[,bws$ixuno, drop = FALSE],class)) ==
-                                class(factor(0)))))
+    if ((any(bws$ixcon) && !all((xccon == "integer") | (xccon == "numeric"))) ||
+        (any(bws$ixord) && !all(sapply(txdat[,bws$ixord, drop = FALSE],inherits, "ordered"))) ||
+        (any(bws$ixuno) && !all(sapply(txdat[,bws$ixuno, drop = FALSE],inherits, "factor"))))
       stop("supplied bandwidths do not match 'txdat' in type")
 
     yccon = unlist(lapply(tydat[,bws$iycon, drop = FALSE],class))
-    if ((any(bws$iycon) && !all((yccon == class(integer(0))) | (yccon == class(numeric(0))))) ||
-        (any(bws$iyord) && !all(unlist(lapply(tydat[,bws$iyord, drop = FALSE],class)) ==
-                                class(ordered(0)))) ||
-        (any(bws$iyuno) && !all(unlist(lapply(tydat[,bws$iyuno, drop = FALSE],class)) ==
-                                class(factor(0)))))
+    if ((any(bws$iycon) && !all((yccon == "integer") | (yccon == "numeric"))) ||
+        (any(bws$iyord) && !all(sapply(tydat[,bws$iyord, drop = FALSE],inherits, "ordered"))) ||
+        (any(bws$iyuno) && !all(sapply(tydat[,bws$iyuno, drop = FALSE],inherits, "factor"))))
       stop("supplied bandwidths do not match 'tydat' in type")
     
     ## catch and destroy NA's
