@@ -10,26 +10,26 @@ npudensbw <- function(...){
 
 npudensbw.formula <-
   function(formula, data, subset, na.action, call, ...){
-    orig.class <- if (missing(data))
-      sapply(eval(attr(terms(formula), "variables"), environment(formula)),class)
-    else sapply(eval(attr(terms(formula), "variables"), data, environment(formula)),class)
+    orig.ts <- if (missing(data))
+      sapply(eval(attr(terms(formula), "variables"), environment(formula)), inherits, "ts")
+    else sapply(eval(attr(terms(formula), "variables"), data, environment(formula)), inherits, "ts")
    
     mf <- match.call(expand.dots = FALSE)
     m <- match(c("formula", "data", "subset", "na.action"),
                names(mf), nomatch = 0)
     mf <- mf[c(1,m)]
 
-    if(all(orig.class == "ts")){
+    if(all(orig.ts)){
       args <- (as.list(attr(terms(formula), "variables"))[-1])
       formula <- terms(formula)
       attr(formula, "predvars") <- as.call(c(quote(as.data.frame),as.call(c(quote(ts.intersect), args))))
       mf[["formula"]] <- formula
-    }else if(any(orig.class == "ts")){
+    }else if(any(orig.ts)){
       arguments <- (as.list(attr(terms(formula), "variables"))[-1])
-      arguments.normal <- arguments[which(orig.class != "ts")]
-      arguments.timeseries <- arguments[which(orig.class == "ts")]
+      arguments.normal <- arguments[which(!orig.ts)]
+      arguments.timeseries <- arguments[which(orig.ts)]
 
-      ix <- sort(c(which(orig.class == "ts"),which(orig.class != "ts")),index.return = TRUE)$ix
+      ix <- sort(c(which(orig.ts),which(!orig.ts)),index.return = TRUE)$ix
       formula <- terms(formula)
       attr(formula, "predvars") <- bquote(.(as.call(c(quote(cbind),as.call(c(quote(as.data.frame),as.call(c(quote(ts.intersect), arguments.timeseries)))),arguments.normal,check.rows = TRUE)))[,.(ix)])
       mf[["formula"]] <- formula
@@ -87,7 +87,11 @@ npudensbw.bandwidth <-
            lbd.dir = 0.1, hbd.dir = 1, dfac.dir = 0.25*(3.0-sqrt(5)), initd.dir = 1.0,
            lbc.init = 0.1, hbc.init = 2.0, cfac.init = 0.5,
            lbd.init = 0.1, hbd.init = 0.9, dfac.init = 0.375,
-           scale.init.categorical.sample=FALSE, ...){
+           scale.init.categorical.sample=FALSE,
+           transform.bounds = FALSE,
+           invalid.penalty = c("baseline","dbmax"),
+           penalty.multiplier = 10,
+           ...){
 
     dat = toFrame(dat)
 
@@ -100,11 +104,9 @@ npudensbw.bandwidth <-
            "'dat'"))
 
     ccon = unlist(lapply(as.data.frame(dat[,bws$icon]),class))
-    if ((any(bws$icon) && !all((ccon == class(integer(0))) | (ccon == class(numeric(0))))) ||
-        (any(bws$iord) && !all(unlist(lapply(as.data.frame(dat[,bws$iord]),class)) ==
-                               class(ordered(0)))) ||
-        (any(bws$iuno) && !all(unlist(lapply(as.data.frame(dat[,bws$iuno]),class)) ==
-                               class(factor(0)))))
+    if ((any(bws$icon) && !all((ccon == "integer") | (ccon == "numeric"))) ||
+        (any(bws$iord) && !all(sapply(as.data.frame(dat[,bws$iord]),inherits, "ordered"))) ||
+        (any(bws$iuno) && !all(sapply(as.data.frame(dat[,bws$iuno]),inherits, "factor"))))
       stop(paste("supplied bandwidths do not match", "'dat'", "in type"))
 
     dat <- na.omit(dat)
@@ -128,6 +130,9 @@ npudensbw.bandwidth <-
     nconfac <- nrow^(-1.0/(2.0*bws$ckerorder+bws$ncon))
     ncatfac <- nrow^(-2.0/(2.0*bws$ckerorder+bws$ncon))                     
 
+    invalid.penalty <- match.arg(invalid.penalty)
+    penalty_mode <- ifelse(invalid.penalty == "baseline", 1L, 0L)
+
     if (bandwidth.compute){
       myopti = list(num_obs_train = dim(dat)[1], 
         iMultistart = ifelse(nmulti==0,IMULTI_FALSE,IMULTI_TRUE),
@@ -160,7 +165,8 @@ npudensbw.bandwidth <-
         old.dens = FALSE,
         int_do_tree = ifelse(options('np.tree'), DO_TREE_YES, DO_TREE_NO),
         scale.init.categorical.sample = scale.init.categorical.sample,
-        dfc.dir = dfc.dir)
+        dfc.dir = dfc.dir,
+        transform.bounds = transform.bounds)
 
       
       myoptd = list(ftol=ftol, tol=tol, small=small,
@@ -178,8 +184,11 @@ npudensbw.bandwidth <-
                          as.integer(myopti), as.double(myoptd), 
                          bw = c(bws$bw[bws$icon],bws$bw[bws$iuno],bws$bw[bws$iord]),
                          fval = double(2), fval.history = double(max(1,nmulti)),
+                         eval.history = double(max(1,nmulti)), invalid.history = double(max(1,nmulti)),
                          timing = double(1),
-                         PACKAGE="np" )[c("bw","fval","fval.history","timing")])[1]
+                         penalty.mode = as.integer(penalty_mode),
+                         penalty.multiplier = as.double(penalty.multiplier),
+                         PACKAGE="np" )[c("bw","fval","fval.history","eval.history","invalid.history","timing")])[1]
       } else {
         nbw = double(ncol)
         if (bws$ncon > 0){
@@ -199,6 +208,8 @@ npudensbw.bandwidth <-
       tbw$fval = myout$fval[1]
       tbw$ifval = myout$fval[2]
       tbw$fval.history <- myout$fval.history
+      tbw$eval.history <- myout$eval.history
+      tbw$invalid.history <- myout$invalid.history
       tbw$timing <- myout$timing
       tbw$total.time <- total.time
     }
@@ -243,6 +254,8 @@ npudensbw.bandwidth <-
                      fval = tbw$fval,
                      ifval = tbw$ifval,
                      fval.history = tbw$fval.history,
+                     eval.history = tbw$eval.history,
+                     invalid.history = tbw$invalid.history,
                      nobs = tbw$nobs,
                      xdati = tbw$xdati,
                      xnames = tbw$xnames,
@@ -269,6 +282,9 @@ npudensbw.default <-
            lbc.init, hbc.init, cfac.init, 
            lbd.init, hbd.init, dfac.init, 
            scale.init.categorical.sample,
+           transform.bounds,
+           invalid.penalty,
+           penalty.multiplier,
            ## dummy arguments for later passing into bandwidth()
            bwmethod, bwscaling, bwtype,
            ckertype, ckerorder, ukertype, okertype,
@@ -311,7 +327,9 @@ npudensbw.default <-
                "lbd.dir", "hbd.dir", "dfac.dir", "initd.dir", 
                "lbc.init", "hbc.init", "cfac.init", 
                "lbd.init", "hbd.init", "dfac.init", 
-               "scale.init.categorical.sample")
+               "scale.init.categorical.sample",
+               "invalid.penalty",
+               "penalty.multiplier")
     m <- match(margs, mc.names, nomatch = 0)
     any.m <- any(m != 0)
 
@@ -326,4 +344,3 @@ npudensbw.default <-
 
     return(tbw)
   }
-
