@@ -12,26 +12,26 @@ npregbw <-
 npregbw.formula <-
   function(formula, data, subset, na.action, call, ...){
 
-    orig.class <- if (missing(data))
-      sapply(eval(attr(terms(formula), "variables"), environment(formula)),class)
-    else sapply(eval(attr(terms(formula), "variables"), data, environment(formula)),class)
+    orig.ts <- if (missing(data))
+      sapply(eval(attr(terms(formula), "variables"), environment(formula)), inherits, "ts")
+    else sapply(eval(attr(terms(formula), "variables"), data, environment(formula)), inherits, "ts")
 
     mf <- match.call(expand.dots = FALSE)
     m <- match(c("formula", "data", "subset", "na.action"),
                names(mf), nomatch = 0)
     mf <- mf[c(1,m)]
 
-    if(all(orig.class == "ts")){
+    if(all(orig.ts)){
       args <- (as.list(attr(terms(formula), "variables"))[-1])
       formula <- terms(formula)
       attr(formula, "predvars") <- as.call(c(quote(as.data.frame),as.call(c(quote(ts.intersect), args))))
       mf[["formula"]] <- formula
-    }else if(any(orig.class == "ts")){
+    }else if(any(orig.ts)){
       arguments <- (as.list(attr(terms(formula), "variables"))[-1])
-      arguments.normal <- arguments[which(orig.class != "ts")]
-      arguments.timeseries <- arguments[which(orig.class == "ts")]
+      arguments.normal <- arguments[which(!orig.ts)]
+      arguments.timeseries <- arguments[which(orig.ts)]
 
-      ix <- sort(c(which(orig.class == "ts"),which(orig.class != "ts")),index.return = TRUE)$ix
+      ix <- sort(c(which(orig.ts),which(!orig.ts)),index.return = TRUE)$ix
       formula <- terms(formula)
       attr(formula, "predvars") <- bquote(.(as.call(c(quote(cbind),as.call(c(quote(as.data.frame),as.call(c(quote(ts.intersect), arguments.timeseries)))),arguments.normal,check.rows = TRUE)))[,.(ix)])
       mf[["formula"]] <- formula
@@ -95,7 +95,11 @@ npregbw.rbandwidth <-
            lbd.dir = 0.1, hbd.dir = 1, dfac.dir = 0.25*(3.0-sqrt(5)), initd.dir = 1.0, 
            lbc.init = 0.1, hbc.init = 2.0, cfac.init = 0.5, 
            lbd.init = 0.1, hbd.init = 0.9, dfac.init = 0.375, 
-           scale.init.categorical.sample = FALSE,...){
+          scale.init.categorical.sample = FALSE,
+          transform.bounds = FALSE,
+          invalid.penalty = c("baseline","dbmax"),
+          penalty.multiplier = 10,
+          ...){
 
     xdat <- toFrame(xdat)
 
@@ -110,11 +114,9 @@ npregbw.rbandwidth <-
       stop("length of bandwidth vector does not match number of columns of 'xdat'")
 
     ccon = unlist(lapply(xdat[,bws$icon, drop = FALSE],class))
-    if ((any(bws$icon) && !all((ccon == class(integer(0))) | (ccon == class(numeric(0))))) ||
-        (any(bws$iord) && !all(unlist(lapply(xdat[,bws$iord, drop = FALSE],class)) ==
-                               class(ordered(0)))) ||
-        (any(bws$iuno) && !all(unlist(lapply(xdat[,bws$iuno, drop = FALSE],class)) ==
-                               class(factor(0)))))
+    if ((any(bws$icon) && !all((ccon == "integer") | (ccon == "numeric"))) ||
+        (any(bws$iord) && !all(sapply(xdat[,bws$iord, drop = FALSE],inherits, "ordered"))) ||
+        (any(bws$iuno) && !all(sapply(xdat[,bws$iuno, drop = FALSE],inherits, "factor"))))
       stop("supplied bandwidths do not match 'xdat' in type")
 
     if (dim(xdat)[1] != length(ydat))
@@ -154,6 +156,9 @@ npregbw.rbandwidth <-
     nconfac <- nrow^(-1.0/(2.0*bws$ckerorder+bws$ncon))
     ncatfac <- nrow^(-2.0/(2.0*bws$ckerorder+bws$ncon))
 
+    invalid.penalty <- match.arg(invalid.penalty)
+    penalty_mode <- ifelse(invalid.penalty == "baseline", 1L, 0L)
+
     if (bandwidth.compute){
       myopti = list(num_obs_train = dim(xdat)[1], 
         iMultistart = ifelse(nmulti==0,IMULTI_FALSE,IMULTI_TRUE),
@@ -188,7 +193,8 @@ npregbw.rbandwidth <-
           ll = REGTYPE_LL),
         int_do_tree = ifelse(options('np.tree'), DO_TREE_YES, DO_TREE_NO),
         scale.init.categorical.sample = scale.init.categorical.sample,
-        dfc.dir = dfc.dir)
+        dfc.dir = dfc.dir,
+        transform.bounds = transform.bounds)
       
       myoptd = list(ftol=ftol, tol=tol, small=small,
         lbc.dir = lbc.dir, cfac.dir = cfac.dir, initc.dir = initc.dir, 
@@ -205,8 +211,11 @@ npregbw.rbandwidth <-
            as.integer(myopti), as.double(myoptd), 
            bw = c(bws$bw[bws$icon],bws$bw[bws$iuno],bws$bw[bws$iord]),
            fval = double(2),fval.history = double(max(1,nmulti)),
+           eval.history = double(max(1,nmulti)), invalid.history = double(max(1,nmulti)),
            timing = double(1),
-           PACKAGE="np" )[c("bw","fval","fval.history","timing")])[1]
+           penalty.mode = as.integer(penalty_mode),
+           penalty.multiplier = as.double(penalty.multiplier),
+           PACKAGE="np" )[c("bw","fval","fval.history","eval.history","invalid.history","timing")])[1]
       
 
       rorder = numeric(ncol)
@@ -216,6 +225,8 @@ npregbw.rbandwidth <-
       tbw$fval <- myout$fval[1]
       tbw$ifval <- myout$fval[2]
       tbw$fval.history <- myout$fval.history
+      tbw$eval.history <- myout$eval.history
+      tbw$invalid.history <- myout$invalid.history
       tbw$timing <- myout$timing
       tbw$total.time <- total.time
     }
@@ -262,6 +273,8 @@ npregbw.rbandwidth <-
                       fval = tbw$fval,
                       ifval = tbw$ifval,
                       fval.history = tbw$fval.history,
+                      eval.history = tbw$eval.history,
+                      invalid.history = tbw$invalid.history,
                       nobs = tbw$nobs,
                       xdati = tbw$xdati,
                       ydati = tbw$ydati,
@@ -290,6 +303,9 @@ npregbw.default <-
            lbc.init, hbc.init, cfac.init, 
            lbd.init, hbd.init, dfac.init,
            scale.init.categorical.sample,
+           transform.bounds = FALSE,
+           invalid.penalty = c("baseline","dbmax"),
+           penalty.multiplier = 10,
            ## dummy arguments for later passing into rbandwidth()
            regtype, bwmethod, bwscaling, bwtype,
            ckertype, ckerorder, ukertype, okertype,
@@ -327,7 +343,10 @@ npregbw.default <-
                "lbd.dir", "hbd.dir", "dfac.dir", "initd.dir", 
                "lbc.init", "hbc.init", "cfac.init", 
                "lbd.init", "hbd.init", "dfac.init", 
-               "scale.init.categorical.sample")
+               "scale.init.categorical.sample",
+               "transform.bounds",
+               "invalid.penalty",
+               "penalty.multiplier")
     m <- match(margs, mc.names, nomatch = 0)
     any.m <- any(m != 0)
 
@@ -343,4 +362,3 @@ npregbw.default <-
     return(tbw)
     
   }
-
