@@ -11,6 +11,8 @@ rbandwidth <-
            fval = NA,
            ifval = NA,
            fval.history = NA,
+           eval.history = NA,
+           invalid.history = NA,
            nobs = NA,
            xdati = stop("rbandwidth:argument 'xdati' missing"),
            ydati = stop("rbandwidth:argument 'ydati' missing"),
@@ -80,6 +82,8 @@ rbandwidth <-
     fval = fval,
     ifval = ifval,
     fval.history = fval.history,
+    eval.history = eval.history,
+    invalid.history = invalid.history,
     scaling = bwscaling,
     pscaling = ifelse(bwscaling, "Scale Factor(s)", "Bandwidth(s)"),
     type = bwtype,
